@@ -154,9 +154,9 @@ destroy_pager(GtkWidget * widget, PagerData *pager)
 }
 
 static const BonoboUIVerb pager_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("PagerProperties", display_properties_dialog),
-	BONOBO_UI_UNSAFE_VERB ("PagerHelp",       display_help_dialog),
-	BONOBO_UI_UNSAFE_VERB ("PagerAbout",      display_about_dialog),
+	BONOBO_UI_UNSAFE_VERB ("PagerPreferences", display_properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("PagerHelp",        display_help_dialog),
+	BONOBO_UI_UNSAFE_VERB ("PagerAbout",       display_about_dialog),
         BONOBO_UI_VERB_END
 };
 
@@ -689,6 +689,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 		xml = glade_xml_new (PAGER_GLADEDIR "/pager.glade", NULL, NULL);
 		pager->properties_dialog = glade_xml_get_widget (xml, "pager_properties_dialog");
 
+		g_object_add_weak_pointer (G_OBJECT (pager->properties_dialog), 
+					   &pager->properties_dialog);
+
 		setup_dialog (xml, pager);
 		
 		g_object_unref (G_OBJECT (xml));
