@@ -26,7 +26,6 @@
 #include "panel-config-global.h"
 #include "session.h"
 #include "status.h"
-#include "swallow.h"
 #include "panel-applet-frame.h"
 
 #define SMALL_ICON_SIZE 20
@@ -47,7 +46,6 @@ static GConfEnumStringPair object_type_enum_map [] = {
 	{ APPLET_DRAWER,   "drawer-object" },
 	{ APPLET_MENU,     "menu-object" },
 	{ APPLET_LOGOUT,   "logout-object" },
-	{ APPLET_SWALLOW,  "swallow-object" },
 	{ APPLET_LAUNCHER, "launcher-object" }, 
 	{ APPLET_EMPTY,    "empty-object" },
 	{ APPLET_LOCK,     "lock-object" },
@@ -167,16 +165,9 @@ applet_idle_remove (gpointer data)
 
 	info->remove_idle = 0;
 
-	if (info->type == APPLET_BONOBO) {
-		PanelAppletFrame *frame = info->data;
-
-		panel_applet_frame_set_clean_remove
-			(PANEL_APPLET_FRAME (frame), TRUE);
-	} else if (info->type == APPLET_SWALLOW) {
-		Swallow *swallow = info->data;
-
-		swallow->clean_remove = TRUE;
-	}
+	if (info->type == APPLET_BONOBO)
+		panel_applet_frame_set_clean_remove (
+				PANEL_APPLET_FRAME (info->data), TRUE);
 
 	panel_applet_clean (info);
 
@@ -229,17 +220,6 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			panel_show_help ("drawers", NULL);
 		}
 		break;
-	case APPLET_SWALLOW:
- 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects", "SWALLOWEDAPP");
-#if 0
-		if(strcmp(menu->name,"properties")==0) {
-			Swallow *swallow = info->data;
-			g_assert(swallow);
-			swallow_properties(swallow); /* doesn't exist yet*/
-		}
-#endif
-		break; 
 	case APPLET_MENU:
 		if(strcmp(menu->name,"properties")==0)
 			menu_properties(menu->info->data);
@@ -259,11 +239,11 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		break;
 	case APPLET_LOCK: {
                 /*
-		  <jwz> Blank Screen Now
-		  <jwz> Lock Screen Now
-		  <jwz> Kill Daemon
-		  <jwz> Restart Daemon
-		  <jwz> Preferences
+		  <jwz> Activate Screensaver
+		  <jwz> Lock Screen
+		  <jwz> Kill Screensaver Daemon
+		  <jwz> Restart Screensaver Daemon
+		  <jwz> Properties
 		  <jwz> (or "configuration" instead?  whatever word you use)
 		  <jwz> those should do xscreensaver-command -activate, -lock, -exit...
 		  <jwz> and "xscreensaver-command -exit ; xscreensaver &"
@@ -653,22 +633,8 @@ applet_do_popup_menu (GtkWidget      *widget,
 	if (panel_applet_in_drag)
 		return FALSE;
 
-	switch (info->type) {
-	case APPLET_BONOBO:
-			break;
-	case APPLET_SWALLOW: {
-		GtkHandleBox *handle_box;
-
-		handle_box = GTK_HANDLE_BOX (((Swallow *)info->data)->handle_box);
-
-		if (handle_box->child_detached)
-			applet_show_menu (info, event);
-		}
-		break;
-	default:
+	if (info->type != APPLET_BONOBO)
 		applet_show_menu (info, event);
-		break;
-	}
 
 	return TRUE;
 }
@@ -785,10 +751,23 @@ typedef struct {
 	PanelWidget *panel_widget;
 	int          position;
 	char        *unique_id;
+	guint        panel_destroy_id;
 } PanelAppletToLoad;
 
 static GSList *panel_applets_to_load = NULL;
 
+static void
+whack_applet_to_load (PanelAppletToLoad *applet)
+{
+	if (applet->panel_destroy_id > 0)
+		g_source_remove (applet->panel_destroy_id);
+	applet->panel_destroy_id = 0;
+	g_free (applet->unique_id);
+	applet->unique_id = NULL;
+	g_free (applet);
+}
+
+
 static gboolean
 panel_applet_load_idle_handler (gpointer dummmy)
 {
@@ -806,12 +785,24 @@ panel_applet_load_idle_handler (gpointer dummmy)
 			applet->position,
 			applet->unique_id);
 
-	g_free (applet->unique_id);
-	g_free (applet);
+	whack_applet_to_load (applet);
 
 	return TRUE;
 }
 
+static void
+panel_destroyed_while_loading (GtkWidget *panel, gpointer data)
+{
+	PanelAppletToLoad *applet = data;
+
+	applet->panel_destroy_id = 0;
+
+	panel_applets_to_load =
+		g_slist_remove (panel_applets_to_load, applet);
+
+	whack_applet_to_load (applet);
+}
+
 static void
 panel_applet_load_from_unique_id (AppletType   type,
 				  GConfClient *gconf_client,
@@ -855,6 +846,11 @@ panel_applet_load_from_unique_id (AppletType   type,
 	panel_widget = panel_widget_get_by_id (panel_id);
 	g_free (panel_id);
 
+	if (!panel_widget) {
+		g_warning ("panel_applet_load: can't find the panel for this object\n");
+		return;
+	}
+
 	/*
 	 * A hack from the old panel to make sure the applet is on the right
 	 */
@@ -869,6 +865,10 @@ panel_applet_load_from_unique_id (AppletType   type,
 		applet->panel_widget = panel_widget;
 		applet->position     = position;
 		applet->unique_id    = g_strdup (unique_id);
+		applet->panel_destroy_id = g_signal_connect
+			(G_OBJECT (panel_widget), "destroy",
+			 G_CALLBACK (panel_destroyed_while_loading),
+			 applet);
 
 		if (!panel_applets_to_load)
 			g_idle_add (panel_applet_load_idle_handler, NULL);
@@ -879,9 +879,6 @@ panel_applet_load_from_unique_id (AppletType   type,
 	case APPLET_DRAWER:
 		drawer_load_from_gconf (panel_widget, position, unique_id);
 		break;
-	case APPLET_SWALLOW:
-		swallow_load_from_gconf (panel_widget, position, unique_id);
-		break;
 	case APPLET_MENU:
 		menu_load_from_gconf (panel_widget, position, unique_id);
 		break;
@@ -1048,10 +1045,6 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 		drawer_save_to_gconf ((Drawer *) applet_info->data,
 				      applet_info->gconf_key);
 		break;
-	case APPLET_SWALLOW:
-		swallow_save_to_gconf ((Swallow *) applet_info->data,
-				       applet_info->gconf_key);
-		break;
 	case APPLET_MENU:
 		menu_save_to_gconf ((Menu *) applet_info->data,
 				    applet_info->gconf_key);
@@ -1143,9 +1136,6 @@ panel_applet_register (GtkWidget      *applet,
 	if (exactpos)
 		insert_at_pos = TRUE;
 
-
-	
-
 	if (panel_widget_add (panel, applet, newpos,
 			      insert_at_pos, expand_major, expand_minor) == -1) {
 		GSList *list;
