@@ -31,7 +31,6 @@
 
 #undef BASEP_WIDGET_DEBUG
 
-extern gboolean panel_applet_in_drag;
 extern GSList *panel_list;
 
 extern int panels_to_sync;
@@ -189,7 +188,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 			      G_STRUCT_OFFSET (BasePWidgetClass, type_change),
 			      NULL,
 			      NULL,
-			      panel_marshal_VOID__ENUM,
+			      g_cclosure_marshal_VOID__ENUM,
 			      G_TYPE_NONE,
 			      1,
 			      PANEL_TYPE_OBJECT_TYPE); */
@@ -212,7 +211,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 				G_STRUCT_OFFSET (BasePWidgetClass, mode_change),
 				NULL,
 				NULL,
-				panel_marshal_VOID__ENUM,
+				g_cclosure_marshal_VOID__ENUM,
 				G_TYPE_NONE,
 				1,
 				PANEL_TYPE_BASE_PMODE);
@@ -224,7 +223,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 				G_STRUCT_OFFSET (BasePWidgetClass, state_change),
 				NULL,
 				NULL,
-				panel_marshal_VOID__ENUM,
+				g_cclosure_marshal_VOID__ENUM,
 				G_TYPE_NONE,
 				1,
 				PANEL_TYPE_BASE_PSTATE);
